package main

import (
	"bufio"
	"os"
	"strings"
	"time"

	"github.com/rs/zerolog"
	"github.com/rs/zerolog/log"
	"github.com/spf13/cobra"

	"github.com/vleeuwenmenno/kcm/src/commands"
	"github.com/vleeuwenmenno/kcm/src/config"
	"github.com/vleeuwenmenno/kcm/src/models"
)

func main() {
	cfg, cfgPath := config.LoadConfig()

	// Set log level
	level, err := zerolog.ParseLevel(cfg.Logging.Level)
	if err != nil {
		level = zerolog.InfoLevel
	}
	zerolog.SetGlobalLevel(level)

	// Set log format
	if cfg.Logging.Format == "console" {
		zerolog.TimeFieldFormat = "[" + time.RFC3339 + "] - "
		log.Logger = log.Output(zerolog.ConsoleWriter{Out: os.Stderr, TimeFormat: time.RFC3339})
	} else {
		zerolog.TimeFieldFormat = zerolog.TimeFormatUnix
	}

	log.Debug().Str("config", cfgPath).Msg("Loaded configuration")

	history := &models.History{MaxItems: cfg.Clipboard.MaxItems}
	history.Load()

	shouldReturn := handleDebugging()
	if shouldReturn {
		return
	}

	var rootCmd = &cobra.Command{
		Use:   "kcm",
		Short: "Clipboard Manager CLI",
	}

	rootCmd.AddCommand(
		commands.NewWatchCmd(history),
		commands.NewListCmd(history),
		commands.NewClearCmd(history),
		commands.NewCopyCmd(history),
		commands.NewSearchCmd(history),
		commands.NewStatusCmd(),
	)

	if err := rootCmd.Execute(); err != nil {
		log.Fatal().AnErr("err", err).Msg("Error executing command")
	}
}

/*
handleDebugging enables interactive debugging mode.
If "debug" is the first argument, prompts for subcommand/args and injects them into os.Args.
Returns true if debugging mode was triggered.
*/
func handleDebugging() bool {
	if len(os.Args) > 1 && os.Args[1] == "debug" {
		zerolog.SetGlobalLevel(zerolog.DebugLevel)
		log.Debug().Msg("Enter the subcommand and arguments you want to run: ")
		reader := bufio.NewReader(os.Stdin)
		command, err := reader.ReadString('\n')
		if err != nil {
			log.Fatal().AnErr("err", err).Msg("Error reading input")
			return false
		}
		command = strings.TrimSpace(command)
		os.Args = append(os.Args[:1], strings.Split(command, " ")...)
		log.Debug().
			Str("input", command).
			Msg("Executing command")
	}
	return false
}
