package models

import (
	"encoding/gob"
	"io"
	"os"
	"os/user"
	"path/filepath"
	"sync"
	"time"

	"github.com/rs/zerolog/log"
	"github.com/vleeuwenmenno/kcm/src/utils"
	"golang.design/x/clipboard"
)

type History struct {
	Items    []HistoryItem
	mu       sync.Mutex
	MaxItems int
}

func historyFilePath() string {
	usr, err := user.Current()
	if err != nil {
		return "./history.gob"
	}
	dir := filepath.Join(usr.HomeDir, ".local", "share", "kcm")
	os.MkdirAll(dir, 0700)
	return filepath.Join(dir, "history.gob")
}

func (h *History) Save() error {
	h.mu.Lock()
	defer h.mu.Unlock()
	file, err := os.Create(historyFilePath())
	if err != nil {
		return err
	}
	defer file.Close()
	enc := gob.NewEncoder(file)
	return enc.Encode(h.Items)
}

func (h *History) Load() error {
	h.mu.Lock()
	defer h.mu.Unlock()
	file, err := os.Open(historyFilePath())
	if err != nil {
		return err
	}
	defer file.Close()
	dec := gob.NewDecoder(file)
	return dec.Decode(&h.Items)
}

func (h *History) Add(item HistoryItem) {
	h.mu.Lock()
	// Prevent duplicates: remove any existing item with the same hash
	itemHash := utils.HashBytes(item.Data)
	var newItems []HistoryItem
	for _, existing := range h.Items {
		if utils.HashBytes(existing.Data) != itemHash {
			newItems = append(newItems, existing)
		} else {
			log.
				Info().
				Str("timestamp", item.Timestamp.Format(time.RFC3339)).
				Msg("Duplicate detected: replaced previous entry with new timestamp.")
		}
	}
	h.Items = newItems
	// Add the new item
	h.Items = append(h.Items, item)
	if h.MaxItems > 0 && len(h.Items) > h.MaxItems {
		over := len(h.Items) - h.MaxItems
		h.Items = h.Items[over:]
	}
	h.mu.Unlock()
	h.Save()
}

func (h *History) Clear() error {
	h.mu.Lock()
	defer h.mu.Unlock()
	h.Items = nil
	return h.Save()
}

func (h *History) List(w io.Writer, noTrunc bool) {
	h.mu.Lock()
	defer h.mu.Unlock()
	for i, item := range h.Items {
		typeStr := "text"
		if item.DataType == clipboard.FmtImage {
			typeStr = "image"
		}
		var msg string
		if noTrunc {
			msg = string(item.Data)
		} else {
			msg = utils.Summary(item.Data, item.DataType)
		}
		log.
			Info().
			Int("index", i+1).
			Str("type", typeStr).
			Str("timestamp", item.Timestamp.Format(time.RFC3339)).
			Msg(msg)
	}
}

func (h *History) ReloadIfChanged() error {
	return h.Load()
}
